#
# The Zope Irc Bot
#
# (The Stupidest Zope Product Ever Written)
#
# (C) Copyright 2000 The Internet (Aust) Pty Ltd
# ACN: 082 081 472  ABN: 83 082 081 472
# All Rights Reserved
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# Author: Andrew Milton <akm@theinternet.com.au>

import App, Globals, OFS
import string
import time

from Globals import ImageFile, HTMLFile, HTML, MessageDialog, package_home
from Acquisition import Implicit
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from App.Extensions import getObject
from webdav.NullResource import NullResource
from ZPublisher.HTTPResponse import HTTPResponse

from Products.IrcBot.Plugins.Plugins import Plugin, PluginRegister
from Products.IrcBot.IrcChannel import IrcChannel

#
# Proposed Options:
#
# Log ChanMsg    -- Default
# Log UserHost   -- Default
# Log PrivMsg
# Log Notice
# Log Ctcp       -- Action Only?
# Log Join
# Log Part
# Log NickChange
# 

def manage_addLogPage(self, id, title='', textIndexerId=''):
	obj=LogPage(id, title, textIndexerId)
	self._setObject(id, obj)
	obj=getattr(self, id)

class LogPage(Folder):

	meta_type="Log Page"
	index_html=HTMLFile('index_html', globals())

	def __init__(self, id, title, textIndexerId):
		self.id = id
		self.title = title
		self.textIndexerId=textIndexerId
		self.logLines=[]

	def addLogLine(self, logLine):
		now=time.strftime("%H:%M", time.localtime(time.time()))
		self.logLines.append(now+' : '+logLine)
		self._p_changed=1
		get_transaction().commit()
		
	def fullBody(self):
		return string.join(self.logLines,'\n')

	def indexPage(self):
		indexer=getattr(self, self.textIndexerId)
		indexer.manage_indexURL(url=self.absolute_url())

def manage_addLogPlugin(self, id, logTypes, textIndexerId='', REQUEST={}):
	""" Add a new Logging Plugin """

	obj=LogPlugin(id=id, logTypes=logTypes, textIndexerId=textIndexerId)
	self._setObject(id, obj)
	obj=getattr(self,id)
	obj.registerBotAction()

	if REQUEST:
		return self.manage_main(self, REQUEST)
	return ''

manage_addLogPluginForm=HTMLFile('manage_addLogPluginForm',globals())

class LogPlugin(Plugin):

	meta_type='Logging Plugin'

	manage_addLogPage=manage_addLogPage
	
	index_html=HTMLFile("LogPlugin_index_html", globals())

	manage_options=(
		{'label':'Configure Plugin', 'action':'manage_configurePluginForm'},
		{'label':'Logs', 'action':'index_html'},
		)
	
	def __init__(self, id, logTypes, textIndexerId=''):
		self.id=id
		self.title="Logging Plugin"
		self.logTypes=logTypes
		self.textIndexerId=textIndexerId
		self.currentPageId=''

	handlers={
		'PRIVMSG':'log_privmsg',
		'CPRIVMSG':'log_cprivmsg',
		'NOTICE':'log_notice',
		'CNOTICE':'log_cnotice',
		'JOIN':'log_join',
		'PART':'log_part',
		'NICK':'log_nick',
		'MODE':'log_mode',
		# 'KICK':'log_kick',
		}

	def registerBotAction(self):
		for x in self.logTypes:
			y = getattr(self, self.handlers[x])
			self.ThreadMethods.addCommand(['BIND', x, y],'^.*')

	def deregisterBotAction(self):
		for x in self.logTypes:
			y = getattr(self, self.handlers[x])			
			self.ThreadMethods.addCommand(['UNBIND', x, y],'^.*')
			
	colours=(
		'#ffffff',
		'#000000',
		'#000080',
		'#008000',
		'#800000',
		'#808000',
		'#800080',
		'#ff0000',
		'#ffff00',
		'#00ff00',
		'#008080',
		'#00ffff',
		'#0000ff',
		'#ff00ff',
		'#404040',
		'#808080')
				
	def text2Html(self, text):
		bold=0
		italic=0
		underline=0
		reverse=0
		colour=0
		BOLD=['</b>', '<b>']
		UNDERLINE=['</u>','<u>']
		REVERSE=['', '']
		COLOUR=['</font>','<font color="%s">']
		s=''
		bgcolour=0
		fgcolour=0
		fg_found=0
		bg_found=0
		finding_colour=0
		for c in text:
			if finding_colour:
				if c==',':
					fg_found=1
					continue
				if c in string.digits:
					if not fg_found: 
						fgcolour=(fgcolour*10)+int(c)
					else:
						bgcolour=(fgcolour*10)+int(c)
					continue
				else:
					if c == '':
						fgcolour=0
						bgcolour=0
						finding_colour=0
						continue
					
					if not fg_found:
						fg_found=1
						bg_found=0
					s=s+COLOUR[colour]%(self.colours[fgcolour])
					fgcolour=0
					bgcolour=0
					finding_colour=0
				
			if c == '': # Bold
				bold=(bold+1)%2
				s=s+BOLD[bold]
				continue
			if c == '': # Reverese
				reverse=(reverse+1)%2
				s=s+REVERSE[reverse]
				continue
			if c == '': # Underline
				underline=(underline+1)%2
				s=s+UNDERLINE[underline]
				continue
			# Colour fg,bg
			# 0 = Bright White ? huh
			# 1 = Black
			# 2 = Blue
			# 3 = Green
			# 4 = Red
			# 5 = Yellow
			# 6 = Purple (Magenta)
			#--------------------- Fg only above here
			# 7 = Bright Red
			# 8 = Bright Yellow
			# 9 = Bright Green
			#10 = Cyan
			#11 = Bright Cyan
			#12 = Bright Blue
			#13 = Bright Purple (Magenta)
			#14 = Grey(50)
			#15 = White
			if c == '': # Color
				colour=(colour+1)%2
				if not colour:
					s=s+COLOUR[colour]
				else:
					finding_colour=1
				continue
			s=s+c
		if bold:
			s=s+BOLD[0]
		if underline:
			s=s+UNDERLINE[0]
		if reverse:
			s=s+REVERSE[0]
		if colour:
			s=s+COLOUR[0]

		return s

	def rotateLogFile(self):
		if not self.currentPageId:
			newId=time.strftime("%Y_%m_%d", time.localtime(time.time()))
			self.manage_addLogPage(id=newId,
								   title=newId,
								   textIndexerId=self.textIndexerId)
			self.currentPageId=newId
			self._p_changed=1
			return

		nowId=time.strftime("%Y_%m_%d", time.localtime(time.time()))
		if self.currentPageId != nowId:
			self.manage_addLogPage(id=nowId, title=nowId,
								   textIndexerId=self.textIndexerId)
			log=getattr(self, self.currentPageId)
			log.indexPage()
			self.currentPageId=nowId
			print "Making New Log: %s"%(self.currentPageId)
			get_transaction().commit()
			self._p_changed=1			

	def addLogLine(self, logLine):
		self.rotateLogFile()
		try:
			log=getattr(self, self.currentPageId)
			log.addLogLine(logLine)			
		except:
			print "We are supposed to have a Log...."


	def log_cprivmsg(self, sender, recip, forum, addl, raw):
		if forum is sender:
			# Private Message to bot...
			return

		msg=self.text2Html(string.rstrip(addl[0][6:]))
		command, args, text=raw
		who=string.split(args[0], '!')[0]				
		logLine="%-20s: *%s*\n"%(who,msg)
		self.addLogLine(logLine)				
		
	def log_privmsg(self, sender, recip, forum, addl, raw):
		if forum is sender:
			# Private Message to bot...
			return
		
		msg=self.text2Html(addl[0])
		command, args, text=raw
		who=string.split(args[0], '!')[0]		
		logLine="%-20s: %s"%(who,msg)
		self.addLogLine(logLine)

	def log_join(self, sender, recip, forum, addl, raw):
		op, args, text = raw
		hostmask=args[0]
		nick,hostmask=string.split(hostmask,'!')
		logLine="*** %s has joined\n"%(nick)
		self.addLogLine(logLine)

	def log_part(self, sender, recip, forum, addl, raw):
		op, args, text = raw
		hostmask=args[0]
		nick,hostmask=string.split(hostmask,'!')
		logLine="*** %s has left\n"%(nick)
		self.addLogLine(logLine)
		
	def log_mode(self, sender, recip, forum, addl, raw):
		pass

	def log_notice(self, sender, recip, forum, addl, raw):
		if forum is sender:
			return
		msg=self.text2Html(string.rstrip(addl[0]))
		command, args, text=raw
		who=string.split(args[0], '!')[0]				
		logLine="%20s: [%s]\n"%(who,msg)
		self.addLogLine(logLine)
		
	def log_cnotice(self, sender, recip, forum, addl, raw):
		pass

	def log_nick(self, sender, recip, forum, addl, raw):
		command, args, text = raw

		who=string.split(args[0], '!')[0]
		text=string.rstrip(text)
		logLine="%s >>>> %s\n"%(who, text)
		self.addLogLine(logLine)
		
LogRegister=PluginRegister('Log', 'Log Channel Events', LogPlugin,
							   manage_addLogPluginForm, manage_addLogPlugin)

IrcChannel.Plugins['Log']=LogRegister
